;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

.include "sprites/assets.inc"
.include "manager/audio.inc"

TRUE = 0
FALSE = 1

.macro breakpoint

    .db 0xED, 0xFF

.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Enable / Disable movement of player
    .macro EnableInput num

        ld hl,#_move_active
        ld (hl),#num

    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Change point of colision in function of status direction 

    .macro getPointofColision
        ld a,e_dir(ix)
        cp #1
        jr nz,check_down_left
            jr calculate_position_colision
        check_down_left:
            cp #-2
            jr nz,endif
        calculate_position_colision:
        ld a,e_posX(ix)
        ld b,e_width(ix)
        dec b
        add b
        ld d,a
        ld a,e_posY(ix)
        ld b,e_height(ix)
        dec b
        add b
        ld e,a
        jr next
        endif:
        ld a,e_posX(ix)
        ld d,a
        ld a,e_posY(ix)
        ld e,a
        next:
        ld a,d
        ld (_pivot_X),a
        ld a,e
        ld (_pivot_Y),a
    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check collision entity with tilemap
    .macro Check_Colisions_Entity_w_Tile

        ld a,e_type(ix)
        cp #e_type_player
        jr nz,no_player
        EnableInput TRUE    ;; Ya ha colisionado, podras moverte
        ;;;;; Comprobamos si son pinchos ;;;;;
        ; A < B ==> A - B < 0
        ld a,#9
        sub (iy)
        jr nc,checkOtherColisions       ;; Si no da signo, es menor
        ; A > B ==> B - A < 0
        ld a,(iy)
        sub #18
        jr nc,checkOtherColisions       ;; si no da signo, es mayor
        
        call man_game_dec_HP
        

        ret
        ; ret ; Salimos del collisions para volver al spawn

        checkOtherColisions:
            
            ld a,(iy)
            cp #9       ;; id meta
            jr z, colision_salida
                cp #78  ;; id chip
                jr nz,no_player
                    
                    ;;nicrochip
                    push hl
                    push iy
                    call man_game_inc_chip
                    pop iy
                    pop hl
                    call man_game_saveChips_map
                
                    ld (iy),#0
                    ret
                    
                jr no_player

        colision_salida:
            ; Pasar de nivel
            
            call man_game_nextLevel
            ret

        no_player:


    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check collision Entity with Portal
    .macro Check_Colision_Entity_W_Portal
        ld a,#53                    ;; if(id_collided > 53)
        sub (iy)
        jr nc,_no_collision_portal
        ld a,(iy)
        sub #66
        jr nc,_no_collision_portal

        ld a,e_type(ix)
        cp #e_type_player
        jr nz,check_portal
        EnableInput FALSE           ;; Desaciva input en caso de que la colision haya sido un portal
        
        check_portal:
        call sys_check_collision_portal

        jp end_collision
        _no_collision_portal:
    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Delete entity if collided with tile
    .macro destroy_Shots

        ld a,e_cmps(ix) 
        and #e_cmp_destroyable
        cp #e_cmp_destroyable
        jr nz,end_collision

        ;se le dice a la entidad que ya puede vovler a realizar su accion (disparar)
        call man_entity_set4destruction
        
        ld l,e_action(ix)
        ld h,e_action+1(ix)         ;cargo en hl el padre
        push hl
        pop iy                      ;convierto a el padre a iy
        ld e_action+1(iy),#0x00     ;carga 00 al padre

    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Get current level
    .macro getLevelSelected_DE
        push hl
        ld hl, (_level_selected)
        ld e,(hl)
        inc hl
        ld d,(hl)
        pop hl

    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check status game to exit
    .macro man_game_checkStatus

        ld a,(_game_STATUS)
        or a
        jr z,continue_game
            cp #1
            jp nz,check_exit
            call man_menu_deathScreen
            jp z,man_game_play_checkpoint
            check_exit:
            cp #2
            jp z,EndGame
            cp #3
            call z,man_menu_WinScreen

        continue_game:

    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Reset chips position
    .macro reset_chips
        
        ld hl,#_mapa_chips
        ld a,(hl)
        cp #0
        jr nz,loop_reset_chips
            cp #0
            jr nz,end_loop_reset
        loop_reset_chips:
            ld e,(hl)
            ld (hl),#0
            inc hl
            ld d,(hl)
            ld (hl),#0
            ld a,#0x4E
            ld (de),a
            inc hl
            ld a,(hl)
            cp #0xFF    
            jr nz,loop_reset_chips
                inc hl
                ld a,(hl)
                dec hl
                cp #0xFF    
                jr z,end_loop_reset

            jr loop_reset_chips
        end_loop_reset:
        ld hl,#_mapa_chips
        ld (_mapa_chips_ptr),hl

        ld hl,#_player_chip+1
        ld (hl),#'0'
        dec hl
        ld (hl),#'0'
        inc hl
        ld de,#_total_chips+1
        ld a,(de)
        add (hl)
        ld (hl),a
        dec hl
        dec de
        ld a,(de)
        add (hl)
        ld (hl),a
        

    .endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Calculate the other exit on portal
    .macro calculatePortalDirection
        ld a,(hl)       ;; Direccion portal
        ld e_dir(ix),a
        inc hl          ;; HL apunta a posicion X
        cp #1
        jr nz,portal_left
            ;; portal_right
            ld a,(hl)         ;; posX portal
            add e_width(ix)
            ld e_posX(ix),a
            inc hl
            ld a,(hl)         ;; posY portal
            ld e_posY(ix),a
            ld e_VX(ix),#1
            ld e_VY(ix),#0
            jp end_calculate_portal_direction

        portal_left:
        cp #-1
        jr nz,portal_up
            ld a,(hl)         ;; posX portal
            sub e_width(ix)
            ld e_posX(ix),a
            inc hl
            ld a,(hl)         ;; posY portal
            ld e_posY(ix),a
            ld e_VX(ix),#-1
            ld e_VY(ix),#0
            jp end_calculate_portal_direction

        portal_up:
        cp #2
        jr nz,portal_down
            ld a,(hl)         ;; posX portal
            ld e_posX(ix),a
            inc hl
            ld a,(hl)         ;; posY portal
            sub e_height(ix)
            ld e_posY(ix),a
            ld e_VX(ix),#0
            ld e_VY(ix),#-4
            jp end_calculate_portal_direction
        portal_down:

            ld a,(hl)         ;; posX portal
            ld e_posX(ix),a
            inc hl
            ld a,(hl)         ;; posY portal
            add e_height(ix)
            ld e_posY(ix),a

            ld e_VX(ix),#0
            ld e_VY(ix),#4

        end_calculate_portal_direction:
        ld a,e_type(ix)
        cp #e_type_player
        call z, man_audio_playPortal


    .endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
