;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Entity_Manager
;##################################################################################
    .area _CODE

    .include "manager/entity.inc"
    .include "macros.inc"
    .include "cpctelera.h.s"


;##################################################################################

_num_entities:: .db 0
_last_elem_ptr:: .dw _entities
_entities::
    .ds MAX_ENTITIES*e_sizeof
.db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Get player on IX
    get_Player_entity_IX::
        ld ix,#_entities
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;   MODIFY A,IX,HL,BC
    man_entity_create::

        ld de,(_last_elem_ptr)
        ld bc,#e_sizeof
        ldir

        ld a,(_num_entities)
        inc a
        ld (_num_entities),a

        ld ix,(_last_elem_ptr)

        ld hl,(_last_elem_ptr)
        ld bc,#e_sizeof
        add hl,bc
        ld (_last_elem_ptr),hl

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;   UPDATE ENTITY
    man_entity_update::
        ld ix,#_entities
        ;; Loop para todas las entidades que no sean invalidas
        entity_update_loop:
            ld a,e_type(ix)
            cp #e_type_invalid
            jr z,entity_update_loop_end 
                cp #e_type_dead
                jr z,destroy_entity
                    ;; Siguiente entidad hasta encontrar una para eliminar o hasta terminar
                    ld bc,#e_sizeof
                    add ix,bc
                    jr entity_update_loop
                destroy_entity:
                    ;; Si esta marcada para borrar, la establecemos como invalida y reorganizamos _entities
                    call man_entity_destroy
                    jr entity_update_loop
        entity_update_loop_end:
        

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;   SET 4 DESTRUCTION ENTITY
    man_entity_set4destruction::
        
        ld e_type(ix),#e_type_dead
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;   DESTROY ENTITY
    man_entity_destroy:
        
        ;; Next free Entity
        ld hl,(_last_elem_ptr)

        ; last entity = nextentity--
        ld bc,#-e_sizeof
        add hl,bc

        ld (_last_elem_ptr),hl
        ld__a_ixl
        sub l
        jr nz,_copy
        ld__a_ixh
        sub h
        jr z,_nocopy
        _copy:
            ; if (last != entity_death)
            ;; [][][][e][][l]
            ;; [][][][l][][l]   ;; copiamos y luego invalidamos la ultima
            ld__e_ixl
            ld__d_ixh
            ld bc,#e_sizeof
            ldir
        _nocopy:
            ;; last = type_invalid
            ld hl,(_last_elem_ptr)
            ld (hl),#e_type_invalid
            
            ld a,(_num_entities)
            sub #1
            ld (_num_entities),a

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UPDATE ENTITY WITH SIGNATURE
    man_entity_forall_matching::
        ld ix,#_entities
        loop_invalid_type:
            ld a,e_type(ix)
            cp #e_type_invalid
            ; if(entity == invalid) return
            ret z
            ;else
            ld a,e_cmps(ix)
            and e
            cp e
            jr nz,endif_e_cmp_signature
                ;if((e->cmps & signature) == signature)

                ; Save signature for nexts entities
                push bc
                push de
                ld hl,#custom_call+1
                ld (hl),c
                inc hl
                ld (hl),b            
                
                call custom_call
                
                pop de
                pop bc
            endif_e_cmp_signature:
            ; Next entity / e++
            
            ; Save BC on stack. custom_call could modify HL
            push bc
            ld bc,#e_sizeof
            add ix,bc
            pop bc
            jr loop_invalid_type
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DESTROY ALL ENTITIES - PLAYER
    man_destroy_all_entities_no_Player::
        ld ix,#_entities
        ld bc,#e_sizeof
        add ix,bc

        loop_invalid_type_2:
            ld a,e_type(ix)
            cp #e_type_invalid
            ; if(entity == invalid) return
            ret z
            ; else

            call man_entity_set4destruction

            add ix,bc

            jr loop_invalid_type_2

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Modify call to custom call
    custom_call:
        call custom_call
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SET DIRECTION SHOT
    ; Input B
    man_entity_calculateShot::

        ld a,e_dir(iy)
        ld e_dir(ix),a

        ld a,e_dir(ix)
        cp #1
        jr nz,check_left
            
            ld a,e_posX(iy)
            add e_width(iy)
            ld e_posX(ix),a

            ld a,e_posY(iy)
            ld e_posY(ix),a

            ; ld hl,#_spr_shot_0
            
            ld e_VX(ix),b

            jr end_check_direction
        check_left:
            cp #-1
            jr nz,check_up

                ld a,e_posX(iy)
                sub e_width(iy)
                ld e_posX(ix),a
                ld a,e_posY(iy)
                ld e_posY(ix),a

                ; ld hl,#_spr_shot_1
                ld a,b
                neg
                ld b,a
                ld e_VX(ix),b
                jr end_check_direction

        check_up:
            cp #2
            jr nz,check_down

                ld a,e_posY(iy)
                sub e_height(iy)
                ld e_posY(ix),a
                ld a,e_posX(iy)
                ld e_posX(ix),a

                ; ld hl,#_spr_shot_3
                ld a,b
                add b           ;; multiplica por 2
                neg
                ld b,a
                ld e_VY(ix),b
                jr end_check_direction
        check_down:
                
                ld a,e_posY(iy)
                add e_height(iy)
                ld e_posY(ix),a
                ld a,e_posX(iy)
                ld e_posX(ix),a

                ; ld hl,#_spr_shot_2
                ld a,b
                add b           ;; multiplica por 2
                ld b,a
                ld e_VY(ix),b
            
        end_check_direction:
            ; ld e_sprite(ix),l
            ; ld e_sprite+1(ix),h

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
