;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Game_Manager
;##################################################################################
    .area _CODE

    .include "cpctelera.h.s"
    .include "cpct_functions.inc"
    .include "manager/entity.inc"
    .include "manager/template.inc"
    .include "system/render.inc"
    .include "system/input.inc"
    .include "manager/animation.inc"
    .include "system/physics.inc"
    .include "system/ai.inc"
    .include "system/collisions.inc"
    .include "manager/menu.inc"
    .include "manager/game.inc"
    .include "system/animation.inc"
    .include "manager/audio.inc"
    .include "macros.inc"
    .globl _main_music
    .globl _win_music

;##################################################################################

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GLOBAL VARIABLES
    _directions: .db 1,-1,2,-2

    _player_hp:: .db 3

    _player_chip:: .db '0','0'
    .db 0

    _mapa_chips: .ds 20
    .dw 0xFFFF

    _mapa_chips_ptr: .dw #_mapa_chips

    ;; LEVELS
                  
    _list_levels: .dw _level1,_level2,_level3,_level4,_level5,_level6,_level7,_level8,_level9
                  .dw _level12,_level13,_level14,_level15,_reference_level,_level16,_level17
                  .dw _level18,_level20,_level21,_level22,_level22_2,_level23,_level23_2,_level24,_level25
                  .dw _level25_2,_level25_3,_level26,_level27,_level27_2,_level28,_level29
    .dw 0xFFFF
    _level_selected:: .dw #_list_levels
    
    ;; CHECKPOINT
    _checkpoint_levels: .dw _level7,_level12,_reference_level,_level23,_level25_3,_level27_2
   _checkpoint_pointer: .dw #_checkpoint_levels
   _checkpoint_saved: .dw #_list_levels
    _contador_SIZE: .dw 0

    ;;Array inicio jugador
    _player_spawn: .dw 0
    _entities_id_spawn:: .db 78,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,0,0xFF
    _posicion_spawn: .db 0,0         ;x, y
    _move_active:: .db 0

    _game_STATUS:: .db -1     ;; 0 - Playing // 1 - Death // 2 - Exit // 3 - FINISH GAME

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SET INIT VALUES
    set_values_0:
        ;; Reset chips
        ld hl,#_player_chip
        ld (hl),#'0'
        inc hl
        ld (hl),#'0'

        ld hl,#_total_chips
        ld (hl),#0
        inc hl
        ld (hl),#0

        ld hl,#_mapa_chips
        ld (_mapa_chips_ptr),hl

        ld hl,#_checkpoint_levels
        ld (_checkpoint_pointer),hl

        ld hl,#_list_levels
        ld (_level_selected),hl
        ld (_checkpoint_saved),hl
        
        
        ld hl,#_time_rendered
        ld (hl),#'0'
        inc hl
        ld (hl),#'0'
        inc hl
        ld (hl),#':'
        inc hl
        ld (hl),#'0'
        inc hl
        ld (hl),#'0'

        ld hl,#_time_temporal
        ld (hl),#0
        inc hl
        ld (hl),#0
        inc hl
        ld (hl),#0
        inc hl
        ld (hl),#0

         



    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PORTALS VAR

    _type_portals::
        .dw _orange_portal
        .dw _purple_portal
        .dw _blue_portal
        .dw 0xFFFF
    _orange_portal::
        .db 0x00            ;; type portal
        .dw 0x0000          ;; posXposY
        .db 0x00
        .dw 0x0000
        .db 0xFD

    _purple_portal:
        .db 0x00
        .dw 0x0000
        .db 0x00
        .dw 0x0000
        .db 0xFD
    _blue_portal: 
        .db 0x00
        .dw 0x0000
        .db 0x00
        .dw 0x0000
        .db 0xFD

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME INIT
;; Creamos entidades, renderizamos mapa, SpawnPlayer 
    man_game_init::
        ;; Creamos la entidad del player
        ld hl, #player_template
        call man_entity_create

        call cpct_akp_stop_asm
        ;; Init music
        ld de,#_game_music
        call cpct_akp_musicInit_asm
        
        ;; Cargamos paleta
        call sys_render_init

        call set_values_0

        jp man_game_play_checkpoint
        
       
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Load Checkpoint Saved
    man_game_play_checkpoint:
        
        ;; Invalidamos entidades anteriores
        call man_destroy_all_entities_no_Player
        call man_entity_update

        ;; Cargamos el nivel que haya en el checkpoint
        ld hl,#_level_selected
        ld de,(_checkpoint_saved)
        ld (hl),e
        inc hl
        ld (hl),d

        ;; Cargamos nuevas entidades
        call man_game_load_level
        ;; Posicionamos al jugador en su spawn correspondiente (Cambia en cada nivel)
        call man_game_getSpawnPlayer
        ;; Eliminar rastro jugador para que el mapa no de fallos
        ld e_previous(ix),#0
        ld e_previous+1(ix),#0
        ;; Si habia chips los vuelve a posicionar
        reset_chips
        ;; Cargamos entidades del nivel
        call sys_render_HUD_screen
        ;; Render tilemap
        call sys_render_tilemap

        ;; Reinicio de valores
        ld a,#3
        ld (_player_hp),a

        ;; Renderizamos el Hud
        call sys_render_HUD_update
        ;; Status: playing
        ld a,#0
        ld (_game_STATUS),a

        ;; Init music
        ld de,#_game_music
        call cpct_akp_musicInit_asm

        ld a,#1
        ld (_render_Time),a
        
        jp man_game_play

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME PLAY
;; Update de todos los sistemas
    man_game_play::
     
        call cpct_waitVSYNC_asm

        call sys_render_update

        ;; Dibujamos todas las entidades
        ;; Actualizacion del estado de entidades
        call man_entity_update
        ;; Actualizacion del comportamiento de enemigos con IA
        call sys_ai_update
        ;; Entrada del teclado
        call sys_input_update
        ;; Actualizamos las fisicas del movimiento de manera alternada para las entidades y reducir velocidad
    
        call sys_physics_update_entities

        ;; Actualizamos constantemente el player
        call sys_physics_update_player

        ;; Comprobamos las colisiones entre entidades y con el tilemap
        call sys_collision_update
        ;; Actualizamos las animaciones de cada entidad
        call sys_animation_update
        
        ;; Render time
        call sys_render_time_update
        
        ;; Comprobamos el estado del juego (Jugando, Game Over, Exit)
        man_game_checkStatus

        jr man_game_play

    EndGame:

    ;; Reset values ;;
    xor a
    ld (_menu_Status),a
    ld a,#-1
    ld (_game_STATUS),a
    call man_destroy_all_entities_no_Player
    call get_Player_entity_IX
    call man_entity_set4destruction
    call man_entity_update

    call cpct_akp_stop_asm

    ld de,#_main_music
    call cpct_akp_musicInit_asm

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SPAWN PLAYER
    man_game_getSpawnPlayer::
        ;; Obtenemos en IX al player
        call get_Player_entity_IX
        push iy
        ld iy,#_player_spawn        
        ld h,0(iy)                  ;; Spawn_X
        ld l,1(iy)                  ;; Spawn_Y
        pop iy

        ld e_posX(ix),h
        ld e_posY(ix),l

        ;; Personaje idle
        ld e_VX(ix),#0
        ld e_VY(ix),#0
        ld e_dir(ix),#0
        
        ;; Habilitamos el input del jugador
        EnableInput TRUE

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DECREASE HP
    man_game_dec_HP::
        ld a,e_dir(ix)
        cp #3                       ;; Si esta en status 5 o 6 no puede quitar mas vida(animacion en proceso)
        ret z
        cp #4
        ret z
        
        ;; Decrementa en 1 la vida
        ld a,(_player_hp)
        dec a
        ld (_player_hp),a
        push ix
        push iy
        ;; Renderizamos el hud
        call sys_render_HP
        pop iy
        pop ix

        EnableInput FALSE    ;; Deshabilitamos Input hasta terminar animacion
        ;; Vuelvo al de antes para ver bien la animacion
        call sys_physics_undo_last_movement
        call man_animation_setDamage

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LOAD LEVEL
    man_game_load_level:
        ;; Reseteamos los portales
        call man_reset_portals
        ;; Reiniciamos el posicionamiento de entidades del mapa
        ld hl,#_posicion_spawn
        ld (hl),#0
        inc hl
        ld (hl),#0
        ;; Nivel actual en DE
        getLevelSelected_DE
        ex de,hl
        ;; Tamano del nivel en BC
        ld bc,#LEVEL_SIZE
    
        ;; Cargamos en contador el tamano para recorrer
        ld (_contador_SIZE),bc
        load_level_loop:            
            
            ;; Contador del mapa
            ld bc,(_contador_SIZE)
            ;; Comprobamos que no ha llegado al final
            ld a,b
            cp #0
            jr nz,continue_level_load
                ld a,c
                cp #0
                ret z           ;; Si los dos bytes son 0, Mapa cargado
            continue_level_load:
            ;; Si no es 0, decrementamos
            dec bc
            ld (_contador_SIZE),bc

            ;; Cargamos el puntero al array de ids conflictivas
            ld iy,#_entities_id_spawn

            ;; Para no comprobar todo, aquellas id que pertenezcan a mapa, pasaran al siguiente
            ;si a es mayor a 54     a - b < 0    b = 54
            ld a, (hl)
            sub #54
            jr c, next_hl
                spawn_id_loop:
                    ld a, (hl)
                    cp (iy)     ;; si es una entidad, la creamos
                    jr nz, next_id
                        call man_game_set_entity
                        jr next_hl
                    next_id:
                        inc iy
                        ld a,(iy)
                        or a
                        jr nz, spawn_id_loop
            next_hl:        ;; Siguiente byte del mapa
                ;; Almacenamos x e y para que cuando haya que crear entidad, sepamos su posicion
                ld a,(hl)
                inc hl
                push hl
                ld hl,#_posicion_spawn
                ld a,(hl)
                cp #20
                jr z, mas_fila
                    ;; Siguiente columna
                    ld a,(hl)
                    inc a
                    ld (hl),a
                    pop hl
                    jr load_level_loop
                mas_fila:
                    ld (hl), #1
                    inc hl
                    ld a,(hl)
                    inc a
                    ld (hl),a
                    pop hl
                jr load_level_loop   

            

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SET ENTITY ON LEVEL
    man_game_set_entity:
        push hl
        ex de,hl

        ;; Cargamos la posicion X e Y que nos encontramos en el mapa (Tiles)
        ld hl,#_posicion_spawn

        ;; Calculamos las coordenadas en pixeles de pantalla
        push af
        ;posX = tx * 4
        ld a,(hl)
        add a
        add a
        ld b,a
        
        ;posY = ty * 8
        inc hl
        ld a,(hl)
        add a
        add a
        add a
        ld c,a

        pop af

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Creacion de PORTALES
            ;;  Si se trata de portales (No entidades), 
            ;;  almacenaremos en sus respectivaas zonas de memoria

            ;; Portal naranja
            cp #58
            jr nc,check_portal_blue
                ;; set_portal_orange
                ld hl,#_orange_portal
                ld a,(hl)
                cp #0               ;; Si ya existe un portal, almacenamos el segundo en el siguiente lugar
                jr nz,next_portal
                    jr set_portal   ;; Guarda portal
 
            ;; Portal Azul
            check_portal_blue:
            cp #62
            jr nc,check_portal_purple
                ;; set_portal_blue
                ld hl,#_blue_portal
                ld a,(hl)
                cp #0
                jr nz,next_portal
                    jr set_portal

            ;; Portal morado
            check_portal_purple:
            cp #66
            jr nc,id_66
                ;; check_portal_purple
                ld hl,#_purple_portal
                ld a,(hl)
                cp #0
                jr nz,next_portal
                    jr set_portal

            next_portal:
                inc hl          ;; Siguiente portal
                inc hl
                inc hl

            set_portal:
                push hl
                push bc
                call man_game_check_direction_portal        ;; output A con direccion de salida
                pop bc
                pop hl
                ld (hl),a       ;; Almacenamos la direccion en la que saldria una entidad por el portal
                inc hl
                ld (hl),b       ;; Almacenamos las posiciones de cada portal para cuando colisione ir a la otra
                inc hl
                ld (hl),c
                jp end_set_entity

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Creación de ENTIDADES
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;; Enemigo movimiento lateral
            id_66:
            cp #66
            jr nz, id_67
                push bc
                ld hl, #enemy_lado_template
                call man_entity_create
                pop bc
                
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #1
                ld e_VX(ix), #1
                jp end_set_entity
                
            id_67:
            cp #67
            jr nz, id_68
                push bc
                ld hl, #enemy_lado_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #-2
                ld e_VY(ix), #1
                jp end_set_entity
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;; Enemigo Disparo
            id_68:
            cp #68
            jr nz, id_69
                push bc
                ld hl, #enemy_shoter_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #1
                jp end_set_entity

            id_69:
            cp #69
            jr nz, id_70
                push bc
                ld hl, #enemy_shoter_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #-1
                jp end_set_entity

            id_70:
            cp #70
            jr nz, id_71
                push bc
                ld hl, #enemy_shoter_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #2
                jp end_set_entity

            id_71:
            cp #71
            jr nz, id_72
                push bc
                ld hl, #enemy_shoter_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #-2
                jp end_set_entity

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;; Enemigo que se alarga
            id_72:
            cp #72
            jr nz, id_73
                push bc
                ld hl, #enemy_long_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_VX(ix) ,#2
                ld e_dir(ix), #1
                ld e_action(ix), b
                ld e_action+1(ix) ,c
                ld e_anim_cnt+1(ix),#72
                ld hl,#_spr_bigshot_0
                ld e_sprite(ix),l
                ld e_sprite+1(ix),h
                jp end_set_entity

            id_73:
            cp #73
            jr nz, id_74
                ; izquierda
                push bc
                ld hl, #enemy_long_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_VX(ix) ,#-2
                ld e_dir(ix), #-1
                ld e_action(ix), b
                ld e_action+1(ix) ,c
                ld e_anim_cnt+1(ix),#73
                ld hl,#_spr_bigshot_1
                ld e_sprite(ix),l
                ld e_sprite+1(ix),h
                jp end_set_entity

            id_74:
            cp #74
            jr nz, id_75
                ;id_26
                push bc
                ld hl, #enemy_long_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_VY(ix) ,#-4
                ld e_dir(ix), #2
                ld e_action(ix), b
                ld e_action+1(ix), c
                ld e_anim_cnt+1(ix),#74
                ld hl,#_spr_bigshot_2
                ld e_sprite(ix),l
                ld e_sprite+1(ix),h
                jp end_set_entity

            id_75:
            cp #75
            jr nz, id_76
                ;id_27
                push bc
                ld hl, #enemy_long_template
                call man_entity_create
                pop bc
                ld e_posX(ix), b
                ld e_posY(ix), c
                ld e_dir(ix), #-2
                ld e_VY(ix) ,#4
                ld e_action(ix), b
                ld e_action+1(ix) ,c
                ld e_anim_cnt+1(ix),#75
                ld hl,#_spr_bigshot_3
                ld e_sprite(ix),l
                ld e_sprite+1(ix),h
                jp end_set_entity

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;; Spawn Player
            id_76:
                cp #76
                jr nz, id_77
                    ld  hl, #_player_spawn
                    ld (hl),b

                    inc hl
                    ld (hl),c
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;; Enemigo hincha
            id_77:
                cp #77
                jr nz,end_set_entity
                    push bc
                    ld hl, #enemy_balloon_template
                    call man_entity_create
                    pop bc
                    ld e_posX(ix), b
                    ld e_posY(ix), c
                    ld e_dir(ix), #1
                    jp end_set_entity
                    
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            end_set_entity:
                pop hl
                ; ld (hl), #0x00
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Save chips position. if level reset
    man_game_saveChips_map::
        
        push iy
        pop de

        ld hl,(_mapa_chips_ptr)
        ld (hl), e
        inc hl
        ld (hl), d

        inc hl
        ld (_mapa_chips_ptr),hl

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Increase chips

    man_game_inc_chip::
        call man_audio_playChip

        ld hl,#_player_chip+1
        ld a,(hl)
        inc a
        cp #':'
        ld (hl),a
        jr nz,render_chip
            ld (hl),#'0'
            dec hl
            inc (hl)
        render_chip:
        call sys_render_numero_chips
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Save chips on checkpoint
    _total_chips: .db 0,0
    man_game_save_chips:
        
        ld hl,#_total_chips+1
        ld de,#_player_chip+1
        ld a,(de)
        sub #48
        ld (hl),a
        dec hl
        dec de
        ld a,(de)
        sub #48
        ld (hl),a

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Load next level
    man_game_nextLevel::
        
        ;; Borrar todas las entidades excepto jugador
        
        call man_destroy_all_entities_no_Player
        call man_entity_update
        
        ;; Cambiamos el nivel seleccionado al siguiente
        
        ld hl,(_level_selected)
        inc hl
        inc hl
        ld (_level_selected),hl

        ld a,(hl)
        cp #0xFF
        jr nz,next_level
            inc hl
            ld a,(hl)
            dec hl
            cp #0xFF
            jr nz,next_level
            ld a,#3
            ld (_game_STATUS),a
            ret


        next_level:

        ld hl,(_level_selected)
        call man_game_check_checkpoint

        call man_game_load_level

        call man_game_getSpawnPlayer

        ld e_previous(ix),#0
        ld e_previous+1(ix),#0

        call sys_render_HUD_screen
        
        call sys_render_tilemap
        
        call sys_render_HUD_update

        ld a,#1
        ld (_render_Time),a
        
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Reset portals Array
    man_reset_portals:
        ld hl,#_type_portals        ;; D.Memoria con los tipos de portales
        ld b,#2                     ;; Reiniciamos 3 veces hasta el 0
        loop_delete_portals:
            ld e,(hl)
            inc hl
            ld d,(hl)           ;; Direccion de memoria tipo portales
            ex de,hl
            loop_delete_same_type:
                ld a,(hl)
                cp #0xFD
                jr nz,delete_portal
                    ex de,hl
                    inc hl
                    ld a,b
                    cp #0
                    ret z
                    dec b
                    jr loop_delete_portals
                delete_portal:
                    ld a,#0
                    ld (hl),a
                    inc hl
            jr loop_delete_same_type

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;d

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check portal direction to launch entity
    man_game_check_direction_portal:
        ;; Primer portal a comprobar
        ld e,#54                ;; Principio portales
        ld d,#66                ;; Terminacion direccion portales
        ld hl,#_directions      ;; Direcciones posibles de un portal
        ld a,e
        loop_check_portal:
        cp (iy)                 ;; comprobamos que el tile que hemos leido
        jr z,set_Portal_Direction
        add #4                  ;; pasamos al siguiente portal con misma direccion
        cp d
        jr c,loop_check_portal
        ;; Pasamos al siguiente tipo de portal
        inc d           
        inc e
        ld a,e
        inc hl
        jr loop_check_portal
        set_Portal_Direction:
        ld a,(hl)
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check if level is chekpoint
    man_game_check_checkpoint:
        
        ex de,hl    ;; DE new level
        ld hl,(_checkpoint_pointer)
        ld a,(de)
        cp (hl)
        ret nz
        inc hl
        inc de
        ld a,(de)
        cp (hl)
        ret nz
        ; save checkpoint and next pointer of list checkpoints
        inc hl
        ld (_checkpoint_pointer),hl
        dec de
        
        ld hl,#_checkpoint_saved
        ld (hl),e
        inc hl
        ld (hl),d

        call man_game_save_chips

        cpctm_clearScreen_asm #0x00

        ld de,#_check_music
        call cpct_akp_musicInit_asm

        call sys_animation_checkpoint

        ld de,#_game_music
        call cpct_akp_musicInit_asm

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Handler Manager
    _count: .db 06
    _contador_tiempo: .db 50
    int_handler::
        push af
        push bc
        push de
        push hl

        ld a,(_count)
        dec a
        jr nz,_nocont

            ;; music play una de cada 6 interrupciones
            call cpct_akp_musicPlay_asm
            call cpct_scanKeyboard_if_asm               ;; interrupt unsafe

            ld a,(_game_STATUS)
            or a
            jr nz,_no_second
            ld a,(_contador_tiempo)
            dec a
            ld (_contador_tiempo),a
            cp #0
            jr nz,_no_second
                ld a,#50
                ld (_contador_tiempo),a
                call man_update_time
            _no_second:
            cp #3
            jr nz,_no_win
                ld a,(_contador_tiempo)
                dec a
                ld (_contador_tiempo),a
                cp #0
                jr nz,_no_win
                ld a,#50
                ld (_contador_tiempo),a

                ld a,(_animation_win_update)
                xor #0x80
                ld (_animation_win_update),a
                
            _no_win:
            
            ld a,#6
        _nocont:
        
        ld (_count),a

        pop hl
        pop de
        pop bc
        pop af

        ei          ;; enable interrupt
        reti        ;; ret interrupt

    set_int_handler::

        call cpct_waitVSYNC_asm
        halt
        halt
        call cpct_waitVSYNC_asm


        ld hl,#0x38
        ld (hl),#0xC3
        inc hl
        ld (hl),#<int_handler
        inc hl
        ld (hl),#>int_handler
        inc hl
        ld (hl),#0xC9
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Update time on game
    _time_temporal: .db 0,0,0,0
    _render_Time: .db 0
    man_update_time:

        ld hl,#_time_rendered+4
        inc (hl)
        ld a,(hl)                   ;; 00:0X
        cp #':'
        jr nz,render_time
        ld (hl),#'0'            
        dec hl
        inc (hl)
        ld a,(hl)                   ;; 00:X0
        cp #'6'
        jr nz,render_time
        ld (hl),#'0'
        dec hl
        dec hl
        inc (hl)
        ld a,(hl)
        cp #':'
        jr nz,render_time
        ld (hl),#'0'
        dec hl
        inc (hl)

        render_time:
        ld a,#1
        ld (_render_Time),a

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
