;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Ai_System
;##################################################################################
    .area _CODE

    .include "manager/entity.inc"
    .include "manager/game.inc"
    .include "manager/template.inc"
    .include "macros.inc"

;##################################################################################


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UPDATE AI
;; Actualiza las entidades que tengan el componente ai
    sys_ai_update::
        ld bc, #sys_ai_update_one_entity
        ld e, #e_cmp_ai
        call man_entity_forall_matching
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UPDATE AI ONE ENTITY
;; Segun la entidad llama a una rutina de ia u otra
    sys_ai_update_one_entity::
        ; Save HL and BC
        push hl
        push bc
        ; Llamada a la rutina especifica segun entidad - especificado el nombre en template
        ld hl,#custom_call_ai+1     
        ld c,e_AI(ix)               ; carga rutina especifica segun template
        ld b,e_AI+1(ix) 
        ld (hl),c
        inc hl
        ld (hl),b
        call custom_call_ai     ;; call bc
        pop bc
        pop hl
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CUSTOM CALL
    custom_call_ai::
        call custom_call_ai
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; IA Enemigo dispara
    sys_ai_behaviour_enemy_shoter::

        ld a,e_action+1(ix)
        cp #0x00
        ret nz

        push ix

        ld e_action+1(ix),#0xFF

        ld hl, #enemy_shot_template
        call man_entity_create

        pop iy

        ld b,#1
        call man_entity_calculateShot

        push iy
        pop de

        ld e_action+1(ix),d
        ld e_action(ix),e
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; IA Long enemy
    sys_ai_behaviour_long::

        dec e_anim_cnt(ix)
        ret nz

        ld a,e_anim_st(ix)
        cp #0
        jr z,create_long
            ;; return_long
            ld e_anim_st(ix),#0

            ld a,e_cmps(ix)
            or #e_cmp_render_raster
            
            jr set_params_long

            create_long:

            ld e_anim_st(ix),#1
            
            ld a,e_cmps(ix)
            xor #e_cmp_render_raster

        set_params_long:

        ld e_cmps(ix),a
        
        ld a,e_VX(ix)
        neg
        ld e_VX(ix),a

        ld a,e_VY(ix)
        neg
        ld e_VY(ix),a

        ld a,e_dir(ix)
        neg
        ld e_dir(ix),a

        ld e_anim_cnt(ix),#0x96

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; IA Enemigo Lateral
    sys_ai_behaviour_enemybasic::

        ld a,e_action(ix)
        cp #0
        ret z
        ld e_action(ix),#0
        ld a,e_VX(ix)
        neg 
        ld e_VX(ix),a
        ld a,e_VY(ix)
        neg 
        ld e_VY(ix),a
        ld a,e_dir(ix)      ; /
        neg                 ;|      Cambia el sentido de la orientacion a su contrario
        ld e_dir(ix),a      ; \
    
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; IA Enemigo Ballon
    .globl sys_render_erase_Sprite  ;; To reset sprite
    sys_ai_behaviour_enemy_balloon::

        ;; e_action actua de forma distinta para cada entidad

        ;; decrementamos el tiempo hasta que llegue a zero
        dec e_action+1(ix)
        ret nz

        ;; Estado de la entidad
        ld a,e_action(ix)
        or a
        ;; Si es 0, debe crecer
        jr z,set_baloon
            ; unset_baloon

            xor a
            ld e_action(ix),a

            ;; Elimina sprite
            call sys_render_erase_Sprite
            ;; Podemos eliminar el sprite aqui (set4destruction) y acontinuacion crear otra entidad
            ;; igual y aprovechar el update de manager para el borrado del sprite grande

            ;; Cambia FORMA
            ld a,e_posX(ix)
            add #4
            ld e_posX(ix),a
            ld a,e_posY(ix)
            add #8
            ld e_posY(ix),a

            ;; Pintar solidbox para borrar

            ld a,#4
            ld e_width(ix),a
            ld a,#8
            ld e_height(ix),a

            ;; Sprite 8x8
            ; ld hl,#_spr_desinflado_0
            ld a,#1
            ld e_dir(ix),a


            jr set_time

        set_baloon:

        ;; Entidad ha crecido
        ld a,#0xFF
        ld e_action(ix),a

        ;; Posicionamos arriba izquierda
        ld a,e_posX(ix)
        sub #4
        ld e_posX(ix),a
        ld a,e_posY(ix)
        sub #8
        ld e_posY(ix),a


        ;; Aumetamos el tamano del enemigo
        ld a,#12
        ld e_width(ix),a
        ld a,#24
        ld e_height(ix),a

        ;; Sprite 24x24
        ; ld hl,#_spr_balloon_11
        ld a,#-1
        ld e_dir(ix),a

        set_time:

        ;; Reinicia contador tiempo (3s)
        ld a,#0x64
        ld e_action+1(ix),a

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
