;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Collisions_System
;##################################################################################
    .area _CODE

    .include "cpctelera.h.s"
    .include "cpct_functions.inc"
    .include "manager/entity.inc"
    .include "system/physics.inc"
    .include "sprites/assets.inc"
    .include "manager/game.inc"
    .include "manager/animation.inc"
    .include "macros.inc"


;##################################################################################

    _pivot_X: .db 0
    _pivot_Y: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COLLISION UPDATE
;; Actualiza y comprueba si colisiona aquellas entidades que tengan el 
;; componente collision y se mueven
    sys_collision_update::
        
        ;; Comprueba de cada entodad que se muueve su colision con el tilemap
        ld bc,#sys_collision_update_one_entity
        ld e,#e_cmp_movable
        call man_entity_forall_matching

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Comprobar colisiones entre entidades

            ; ld ix,#_entities        ;; primera entidad

            ; ld iy,#_entities

            ; ld bc,#e_sizeof

            ; next_ix:

            ;     ld a,e_type(ix)
            ;     cp #e_type_invalid
            ;     ret z

            ;     push ix
            ;     next_iy:

            ;     add iy,bc

            ;     ld a,e_type(iy)
            ;     cp #e_type_invalid
            ;     jr z,add_ix_and_next

            ;     call sys_collision_check

            ;     jr next_iy

            ; add_ix_and_next:
            ;     pop ix
            ;     add ix,bc
            ;     push ix
            ;     pop iy
            ;     jr next_ix

            ; ret
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Comprobar colisiones solo con Player
            
            ld ix,#_entities        ;; primera entidad (Player)
            ld iy,#_entities
            ld bc,#e_sizeof
            add iy,bc

            next_iy:

                ld a,e_type(iy)
                cp #e_type_invalid
                ret z

                call sys_collision_check

                add iy,bc

                jr next_iy

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COLLISION UPDATE ONE ENTITY
    sys_collision_update_one_entity:

        xor a
        ;; Calculamos la posicion de la entidad en el tilemap
        call getPosition_on_tile

        ; Devuelve IY, byte del mapa en el que esta la entidad

        ;Si existe colision continua
        ld hl,#_entities_id_spawn+13        ;; + 13 colisiones que no interesan

        ;; Bucle para evitar colisionar con el spawn de cada entidad
        loop_no_spawn_collision:
            ld a,(hl)
            cp (iy)
            jr z,no_collision
            cp #0xFF
            jr z,_collision
            inc hl
            jr loop_no_spawn_collision

        no_collision:
        ret
        _collision:
        
        ;Comprueba si el tile de colision
        Check_Colisions_Entity_w_Tile
        
        ;; Comprobar la franja de id portales
        Check_Colision_Entity_W_Portal
        
        ;volver a la posicion anterior del jugador para no meterse en la pared
        call sys_physics_undo_last_movement
        
        continue_collision:
        ;si la entidad que ha chocado es destruible se elimina (bala)
        destroy_Shots

        end_collision:
        ld a,e_type(ix)
        cp #e_type_enemy_lado
        ret nz
        ld e_action(ix),#0xFF       ;detecta colision (lado a lado)

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GET POSITION ON TILE
;; Posicion de la entidad con respecto al tilmeap

    ; ADRESS = BASE + Y*20 + X
    ;         |    | | OFFSET |

    ; Calculate player position on tile
    ; TileX => 4*TX = posX
    ; TileY => 8*TY = posY
    getPosition_on_tile:

        ;Comprobar si esta alineado con el tilemap (si tiene resto no esta alineado)
        getPointofColision
        
        ;Si esta alineado se comprueba si hay colision con el tilemap

        ; Step 1: Calculate Entity.x / 4
        ; A = entity.x

        ld a,d
        ;ld a,e_posX(ix) ; A = Entity.x
        srl a           ; A = Entity.x / 2
        srl a           ; A = Entity.x / 4
        ; Save A on C
        ld c,a          ; C = Entity.x / 4
        ld b,#0
        ; Step 2: Calculate Entity.y / 8
        ld a,e
        ;ld a,e_posY(ix) ; A = Entity.y          ;; 0b0001'0110
        ; to delete residue and solution of division
        and #0b11111000 ; A = 8*(Entity.y / 8)  ;; 0b0001'0000 --110
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;::::::::::::::::
        ; srl a           ; A = Entity.y / 2      ;; 0b0000'1011 0
        ; srl a           ; A = Entity.y / 4      ;; 0b0000'0101 10
        ; srl a           ; A = Entity.y / 8      ;; 0b0000'0010 110
        ; sll a           ; A = 2*(Entity.y / 8)  ;; 0b0000'0100 110
        ; sll a           ; A = 4*(Entity.y / 8)  ;; 0b0000'1000 110
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;::::::::::::::::
        ld e,a              ;; / 
        srl e               ;;| DE = 4*int(Entity.y / 8)
        ld d,b              ;; \ 
        ld l,a          
        ld h,d              ;; \ HL = A = 8*int(Entity.y / 8)
        add hl,hl           ;;   HL =    16*int(Entity.y / 8)
        add hl,de           ;;   HL =    20*int(Entity.y / 8)
        add hl,bc           ;;   HL =    20*int(Entity.y / 8) + (Entity.x / 4)
        ex de,hl            ;;   DE =    20*int(Entity.y / 8) + (Entity.x / 4
        push de
        getLevelSelected_DE
        push de
        pop iy ;; IY = BASE ADRESS
        pop de
        add iy, de          ;; IY = BASE + OFFSET
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; IX = ENTITY 1
;; IY = ENTITY 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CHECK COLLISIONS BETWEEN ENTITIES
    sys_collision_check::
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Check collisions long entity enemy
            ld a,e_type(iy)
            cp #e_type_enemy_long
            jp nz,_no_long_colision
                ld a,e_type(ix)
                cp #e_type_player
                jr nz,_no_long_colision
                ld a,e_anim_cnt+1(iy)
                cp #73
                jr z,left_up
                    cp #74
                    jr z,left_up
                right_down:
                ld a,e_posX(iy)
                add e_width(iy)
                dec a
                cp e_posX(ix)
                jp c,__no_collision


                ld a,e_posX(ix)
                add e_width(ix)
                dec a
                cp e_action(iy)
                jp c,__no_collision


                ld a,e_posY(iy)
                add e_height(iy)
                dec a
                cp e_posY(ix)
                jp c,__no_collision

                ld a,e_posY(ix)
                add e_height(ix)
                dec a
                cp e_action+1(iy)
                jp c,__no_collision
                
                jp long_collided
                left_up:
                
                ld a,e_posY(ix)
                add e_height(ix)
                dec a
                cp e_posY(iy)
                jr c,__no_collision

                ld a,e_action+1(iy)
                add e_height(iy)
                dec a
                cp e_posY(ix)
                jr c,__no_collision
                
                ld a,e_posX(ix)
                add e_width(ix)
                dec a
                cp e_posX(iy)
                jr c,__no_collision

                ld a,e_action(iy)
                add e_width(iy)
                dec a
                cp e_posX(ix)
                jr c,__no_collision
                

                long_collided:
                call man_game_dec_HP

            ret
            _no_long_colision:
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Check collisions with normal entity
            ; if( e_posX(ix) + e_width(ix) - e_posX(iy) < 0) __no_collision
            ld a,e_posX(ix)
            add e_width(ix)
            dec a       ;; si no decrementamos, colisiona una al lado de la otra
            sub e_posX(iy)
            jr c,__no_collision


            ; if( e_posX(iy) + e_width(iy) - e_posX(ix) < 0) __no_collision
            ld a,e_posX(iy)
            add e_width(iy)
            dec a       ;; si no decrementamos, colisiona una al lado de la otra
            sub e_posX(ix)
            jr c, __no_collision

            ; if( e_posY(ix) + e_height(ix) - e_posY(iy) < 0) __no_collision
            ld a,e_posY(ix)
            add e_height(ix)
            dec a       ;; si no decrementamos, colisiona una al lado de la otra
            sub e_posY(iy)
            jr c, __no_collision

            ; if( e_posY(iy) + e_height(iy) - e_posY(ix) < 0) __no_collision
            ld a,e_posY(iy)
            add e_height(iy)
            dec a       ;; si no decrementamos, colisiona una al lado de la otra
            sub e_posY(ix)
            jr c, __no_collision
            
            ld a,e_type(ix)
            cp #e_type_player
            ret nz
            
            call man_game_dec_HP

            __no_collision:
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


        ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check if entity collided with which portal
    sys_check_collision_portal:
        
        ld hl,#_type_portals
        
        loop_portal_type:

        ld e,(hl)
        inc hl
        ld d,(hl)           ;; Direccion de memoria tipo portales


        ex de,hl            ;; Queremos usar HL para recorrer valores portales

        inc hl              ;; Incrementamos para acceder a posX portal
        push hl

        ;; Comprobamos que portal es el colisionado
        ;;;;;;;;;;;
            loop_portales:

            ld a,(_pivot_X)
            ld b,a
            ld a,(_pivot_Y)
            ld c,a

            ld a,(hl)
            cp b
            jr z,check_Y               ;; x <= xTile
            jr nc,next_portal_same_type

            check_max_X:
            add #4
            cp b
            jr c,next_portal_same_type

            check_Y:
            inc hl
            ld a,(hl)
            cp c
            jr z,check_max_Y               ;; x <= xTile
            jr nc,next_portal_same_type

            check_max_Y:
            add #8
            cp c
            jr nc,set_portal

        ;;;;;;;;;;

        next_portal_same_type:
        pop hl
        inc hl
        inc hl
        ld a,(hl)
        cp #0xFD
        jr z,next_portal_type           ;; Siguiente portal
            inc hl
            push hl
            jp loop_portales
        next_portal_type:       ;; Pasamos al puntero nuevo de portales de tipo distinto
        ex de,hl
        inc hl
        ld a,(hl)
        cp #0xFF
        jp nz,loop_portal_type
        inc hl
        ld a,(hl)
        cp #0xFF
        ret z
        dec hl
        ;; New portal type
        jp loop_portal_type

        set_portal:
        pop hl
        inc hl
        inc hl
        ld a,(hl)
        cp #0xFD
        jr z,first_portal
            ;; Ir a portal 2
            jp calculate_Direction
            
        first_portal:
        ld bc,#-6
        add hl,bc
        
        calculate_Direction:

        calculatePortalDirection

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
