;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Input_System
;##################################################################################
    .area _CODE

    .include "cpctelera.h.s"
    .include "cpct_functions.inc"
    .include "manager/entity.inc"
    .include "manager/game.inc"
    .include "manager/menu.inc"
    .include "manager/audio.inc"
    .include "macros.inc"

;##################################################################################

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT UPDATE
    sys_input_update::
        
        xor a
        call get_Player_entity_IX

        ;; Input solo para player

        call sys_input_check_keyboard

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input WAIT Key Released
;;      RET A -> 1 // can pressed // A -> 0 // pressed,wait to release
    sys_input_check_Key_Released:
        ; call cpct_scanKeyboard_f_asm    ;; Cuando suelte la tecla, entra al metodo de seleccionar KEY
        ld a,(_key_pressed)
        or a
        jr nz,pressed
            call cpct_isAnyKeyPressed_asm
            or a
            jr z,key_pressed_wait
            ;; TRUE
            ld a,#1
            ld (_key_pressed),a
            ret
        pressed:
            call cpct_isAnyKeyPressed_asm
            or a
            jr nz,key_pressed_wait
                ld a,#0
                ld (_key_pressed),a
                ret
        key_pressed_wait:
        ld a,#0
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_key_pressed: .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Main
    sys_input_key_menu::

        call sys_input_check_Key_Released
        or a
        ret z
        ld iy,#key_actions_menu-4
        call check_input

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input in Game
    sys_input_check_keyboard::
        
        call sys_input_check_Key_Released
        or a
        ret z

        ld iy,#key_actions-4

        ld a,(_move_active)         ;; Comprobamos que se haya habilitado el uso de input (move_active == 0)
        cp #1
        ret z

        call check_input

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Key Actions IN GAME

    key_actions:
        .dw Key_Q, action_to_up
        .dw Joy0_Up,action_to_up

        .dw Key_O, action_to_left
        .dw Joy0_Left,action_to_left

        .dw Key_A, action_to_down
        .dw Joy0_Down,action_to_down

        .dw Key_P, action_to_right
        .dw Joy0_Right,action_to_right

        .dw Key_Esc,go_to_menu
        .dw 0


    action_to_right:
        
        ld e_dir(ix),#1
        ld e_VX(ix),#1
        ld e_VY(ix),#0
        EnableInput FALSE
        call man_audio_playMove

    ret
    action_to_left:
        ld e_dir(ix),#-1
        ld e_VX(ix),#-1
        ld e_VY(ix),#0
        EnableInput FALSE
        call man_audio_playMove

    ret
    action_to_up:
        ld e_dir(ix),#2
        ld e_VX(ix),#0
        ld e_VY(ix),#-4
        EnableInput FALSE
        call man_audio_playMove

    ret
    action_to_down:
        ld e_dir(ix),#-2
        ld e_VX(ix),#0
        ld e_VY(ix),#4
        EnableInput FALSE
        call man_audio_playMove
        
    ret

    go_to_menu:
        ld a,#3
        ld (_game_STATUS),a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_render_menu:: .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Key Actions MENU

    key_actions_menu:
        .dw Key_O, menu_to_left
        .dw Joy0_Left,menu_to_left

        .dw Key_P, menu_to_right
        .dw Joy0_Right,menu_to_right

        .dw Key_Q, menu_to_up
        .dw Joy0_Up, menu_to_up

        .dw Key_A, menu_to_down
        .dw Joy0_Down, menu_to_down

        .dw Key_Space, enter_menu
        .dw Joy0_Fire1, enter_menu
        .dw 0

    menu_to_left:
    
        call menu_to_up_guide

        ld a,(_menu_Status)
        cp #2                               ;: Si no es guide no cambia paginas
        ret nz    

    ret

    menu_to_right:

        call menu_to_down_guide  

        ld a,(_menu_Status)
        cp #2                               ;; Si no es controls no cambia paginas
        ret nz
        
    ret

    menu_to_up:
        ld a,(_menu_Status)
        cp #2                               ;: bloqueamos para controls
        ret z
        menu_to_up_guide:
        call man_audio_playSelect
        ld a,#1
        ld (_render_menu),a

        ld a,(_menu_selected)
        dec a
        cp #0
        ld (_menu_selected),a
        ret nz
        ld a,#1
        ld (_menu_selected),a


    ret

    menu_to_down:

        ld a,(_menu_Status)
        cp #2                               ;: bloqueamos para guide
        ret z

        menu_to_down_guide:
        call man_audio_playSelect
        ld a,#1
        ld (_render_menu),a
        ld a,(_menu_selected)
        inc a
        cp #5
        ld (_menu_selected),a
        ret nz
        ld a,#4
        ld (_menu_selected),a

    ret


    enter_menu:
        ld a,(_menu_selected)
        ld (_menu_Status),a
        ld a,#0xFF
        ld (_menu_selected),a

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CHECK KEY PRESSED
    ;; recorriendo un registro de teclas
    check_input:
        loop_keys:

        ; Next key / Key++
        ld bc,#4
        add iy,bc

        ; HL = KEY
        ld l,0(iy)
        ld h,1(iy)
        ; If(A == 0, Key=null)
        ld a,l
        or h
        ret z

        ; if key HL is pressed
        call cpct_isKeyPressed_asm
        jr z,loop_keys

        ld hl,#loop_keys
        push hl
        ld l,2(iy)
        ld h,3(iy)
        jp (hl)     ;; Saltamos a la letra pulsada y al metodo correspondiente

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
