;##################################################################################################################
;##################################################################################################################
; Copyright (c) 2022 MedusaProjects | Quico Blázquez Vidal | Lucila Vázquez Soriano | Alexandru Carlos Sular Feier

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;##################################################################################################################
;##################################################################################################################

;##################################################################################
.module Render_System
;##################################################################################
    .area _CODE

    .include "cpctelera.h.s"
    .include "cpct_functions.inc"
    .include "manager/entity.inc"
    .include "manager/game.inc"
    .include "sprites/assets.inc"
    .include "manager/menu.inc"
    .include "macros.inc"
    .include "screens/screenmenu_z.h.s"
    .include "screens/screenoption_z.h.s"
    .include "screens/screenhud_z.h.s"

    
    .globl _game_palette

    .globl _menu_palette


;##################################################################################

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER INIT
    sys_render_init::
        ; Set palette
        ld hl, #_game_palette
        ld de, #16
        call cpct_setPalette_asm

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER UPDATE
;; Actualiza todas las entitades que tengan componente render
    sys_render_update::
        ;; Cargamos la rutina que hara cada entidad
        ld bc, #sys_render_one_entity
        ;; Debe tener el componente render
        ld e, #e_cmp_render
        call man_entity_forall_matching
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER ONE ENTITY
    sys_render_one_entity::
        ;; Comprobamos si hay que dejar rastro o no
        ld e,#e_cmp_render_raster
        ld a,e_cmps(ix)
        and e
        cp e
        jr z,_dont_clean_previous
            ;; Clean entity from previous location when needed draw new position
            ;; if (e_previous != 0x00--) clear_previous_position
            ld a,e_previous+1(ix)
            or a
            jr z,_dont_clean_previous
            ;; si la entidad se mueve borramos el rastro, su posicion anterior (previous)
        _clear_previous_position:
            ld d,a
            ld e,e_previous(ix)
            ld c,e_width(ix)
            ld b,e_height(ix)
            xor a
            call cpct_drawSolidBox_asm

        _dont_clean_previous:

        ld de,#CPCT_VMEM_START_ASM
        ld c,e_posX(ix)
        ld b,e_posY(ix)
        call cpct_getScreenPtr_asm

        ;Actualizamos previous - pasa a ser la posicion actual
        ld e_previous(ix),l
        ld e_previous+1(ix),h

        ex de, hl
        ld c, e_width(ix)
        ld b, e_height(ix)

        ;; if (e_type == dead)
        ld a,e_type(ix)
        cp #e_type_dead
        jr z,endif_drawSolidBox

        ;; Si la entidad no esta muerta la dibujamos en su nueva posicion
        ld l, e_sprite(ix)
        ld h,e_sprite+1(ix)
        call cpct_drawSprite_asm

        ret

        ;; Si la entidad esta muerta borrarla con SolidBox
        endif_drawSolidBox:
        
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER HUD   
    sys_render_HUD_update::
        call sys_render_HP
        call sys_render_numero_chips
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER HUD SCREEN
    sys_render_HUD_screen::
        ld hl,#_screenhud_z_end
        ld de,#0xFFFF
        call cpct_zx7b_decrunch_s_asm
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TIME RENDERES
    _time_rendered:: .db '0','0',':','0','0',0
    sys_render_time::

        ld hl,#0x000F
        call cpct_setDrawCharM0_asm

        ld iy,#_time_rendered

        ld hl,#0xD701

        call cpct_drawStringM0_asm

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Render Time on game

    sys_render_time_update::

        ld a,(_render_Time)
        or a
        jr z,no_render_time
            dec a
            ld (_render_Time),a
            call sys_render_time
        no_render_time:


    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER HEARTS
    sys_render_HP::

        call get_Player_entity_IX

        ld de,#CPCT_VMEM_START_ASM
        ld c,#7
        ld b,#174
        call cpct_getScreenPtr_asm

        ex de,hl                ;; donde dibujo

        ld a,(_player_hp)
        
        loop_HP:

            cp #0
            jr z,end_loop_hp

            ld hl,#_spr_heart_0     ;; sprite a dibujar
            ld c,#SPR_HEART_W
            ld b,#SPR_HEART_H
            push de
            push af
            call cpct_drawSprite_asm
            pop af
            pop de
            ex de,hl
            ld de,#8
            add hl,de
            ex de,hl
            
            dec a
            
            jr loop_HP

        end_loop_hp:

            ld a,#MAX_HP_PLAYER
            push hl
            ld hl,#_player_hp
            sub (hl)
            pop hl
            loop_HP_empty:

                cp #0
                ret z

                ld hl,#_spr_heart_1     ;; sprite a dibujar
                ld c,#SPR_HEART_W
                ld b,#SPR_HEART_H
                push de
                push af
                call cpct_drawSprite_asm
                pop af
                pop de
                ex de,hl
                ld de,#8
                add hl,de
                ex de,hl

                dec a
            
            jr loop_HP_empty

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Render number of chips
    sys_render_numero_chips::

        ld hl,#0x0004
        call cpct_setDrawCharM0_asm

        ld   iy, #_player_chip

        ld de,#CPCT_VMEM_START_ASM
        ld c,#56
        ld b,#177
        call cpct_getScreenPtr_asm

        call cpct_drawStringM0_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER TILEMAP   c, b, de, hl
;; Dibuja el mapa .tmx
    sys_render_tilemap::
        ; Dimension del tilemap - 20x20 en hexadecimal
        ld bc, #0x1414
        ld de, #20
        ld hl, #_tiles0_00
        call cpct_etm_setDrawTilemap4x8_ag_asm

        ;Posicion direccion de memoria donde empezar a dibujar el mapa, no puede ser cualqueira - restricciones de alineado - coincidir con linea 0
        ld hl, #CPCT_VMEM_START_ASM
        getLevelSelected_DE
        call cpct_etm_drawTilemap4x8_ag_asm

    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Render MENUS
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Menu Game Over
        _yes_text: .asciz "Yes"
        _no_text: .asciz "No"

        sys_render_menu_death::
            ld hl,#0x0004
            ld a,(_menu_selected)
            cp #1
            call z,set_color_death

            call cpct_setDrawCharM0_asm
            ld iy,#_yes_text

            ld de,#CPCT_VMEM_START_ASM
            ld c,#20
            ld b,#100
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm
            
            ld hl,#0x0004
            ld a,(_menu_selected)
            cp #2
            call z,set_color_death

            call cpct_setDrawCharM0_asm
            ld iy,#_no_text

            ld de,#CPCT_VMEM_START_ASM
            ld c,#52
            ld b,#100
            call cpct_getScreenPtr_asm
            
            call cpct_drawStringM0_asm

        ret

        set_color_death:
            ld hl,#0x000F
        ret

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;Render Credits
        sys_render_menu_credits::


            ld hl,#0x060F
            call cpct_setDrawCharM0_asm
            ld iy,#credit_intro

            ld de,#CPCT_VMEM_START_ASM
            ld c,#CREDITINTRO_X
            ld b,#CREDITINTRO_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ld hl,#0x060D
            call cpct_setDrawCharM0_asm
            ld iy,#credit_text3

            ld de,#CPCT_VMEM_START_ASM
            ld c,#CREDIT3_X
            ld b,#CREDIT3_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm


            ld hl,#0x060D
            call cpct_setDrawCharM0_asm
            ld iy,#credit_text1

            ld de,#CPCT_VMEM_START_ASM
            ld c,#CREDIT1_X
            ld b,#CREDIT1_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm


            ld hl,#0x060D
            call cpct_setDrawCharM0_asm
            ld iy,#credit_text2

            ld de,#CPCT_VMEM_START_ASM
            ld c,#CREDIT2_X
            ld b,#CREDIT2_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ld hl,#0x0607
            call cpct_setDrawCharM0_asm
            ld iy,#credit_text4

            ld de,#CPCT_VMEM_START_ASM
            ld c,#CREDIT4_X
            ld b,#CREDIT4_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

        ret


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Menu Principal
        sys_render_main_screen::
            ; Set palette
            ld hl, #_menu_palette
            ld de, #16
            call cpct_setPalette_asm

            ld hl,#_screenmenu_z_end
            ld de,#0xFFFF

            call cpct_zx7b_decrunch_s_asm
        ret
        sys_render_menu_principal::
            
            ; Primero renderizamos todas las letras y luego pintamos por encima la que queremos
            ld hl,#0x060D
            ld a,(_menu_selected)
            cp #1
            call z,set_color
            call cpct_setDrawCharM0_asm
            ld iy,#init_text

            ld de,#CPCT_VMEM_START_ASM
            ld c,#TEXT1_X
            ld b,#TEXT1_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm


            ld hl,#0x060D
            ld a,(_menu_selected)
            cp #2
            call z,set_color
            call cpct_setDrawCharM0_asm
            ld iy,#init_text2

            ld de,#CPCT_VMEM_START_ASM
            ld c,#TEXT2_X
            ld b,#TEXT2_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ld hl,#0x060D
            ld a,(_menu_selected)
            cp #3
            call z,set_color
            call cpct_setDrawCharM0_asm
            ld iy,#init_text3

            ld de,#CPCT_VMEM_START_ASM
            ld c,#TEXT3_X
            ld b,#TEXT3_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ld hl,#0x060D
            ld a,(_menu_selected)
            cp #4
            call z,set_color
            call cpct_setDrawCharM0_asm
            ld iy,#init_text4

            ld de,#CPCT_VMEM_START_ASM
            ld c,#TEXT4_X
            ld b,#TEXT4_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm
            


        ret

        set_color:
            ld hl,#0x060F
        ret
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Menu Otions Zoom
        sys_render_option_screen::
            ; Set palette
            ld hl, #_menu_palette
            ld de, #16
            call cpct_setPalette_asm

            ld hl,#_screenoption_z_end
            ld de,#0xFFFF

            call cpct_zx7b_decrunch_s_asm
        ret

        sys_render_menu_control::

            ld hl,#0x060F
            call cpct_setDrawCharM0_asm
            ld iy,#init_text2

            ld de,#CPCT_VMEM_START_ASM
            ld c,#TEXT5_X
            ld b,#TEXT5_Y
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

        ret

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    
    _win_text1: .asciz "-- TIME OUT --"
    _win_text3: .asciz  "Chips"
    _win_text4: .asciz  "Share with:"
    _win_text5: .asciz  "#MorrisScore"

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Win screen
        sys_render_win_screen::
            ;; TIME
            ld hl,#0x060F
            call cpct_setDrawCharM0_asm

            ld  iy,#_win_text1

            ld de,#CPCT_VMEM_START_ASM
            ld c,#12
            ld b,#30
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Chips text

            ld hl,#0x060F
            call cpct_setDrawCharM0_asm

            ld  iy,#_win_text3

            ld de,#CPCT_VMEM_START_ASM
            ld c,#30
            ld b,#60
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Chips num

            ld hl,#0x060F
            call cpct_setDrawCharM0_asm

            ld  iy,#_player_chip

            ld de,#CPCT_VMEM_START_ASM
            ld c,#36
            ld b,#80
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Time Finished num

            ld hl,#0x060F
            call cpct_setDrawCharM0_asm

            ld  iy,#_time_rendered

            ld de,#CPCT_VMEM_START_ASM
            ld c,#30
            ld b,#110
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Share text

            ld hl,#0x060F
            call cpct_setDrawCharM0_asm

            ld  iy,#_win_text4
            ld de,#CPCT_VMEM_START_ASM
            ld c,#20
            ld b,#140
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Hashtag text

            ld hl,#0x0607
            call cpct_setDrawCharM0_asm

            ld  iy,#_win_text5
            ld de,#CPCT_VMEM_START_ASM
            ld c,#16
            ld b,#160
            call cpct_getScreenPtr_asm

            call cpct_drawStringM0_asm

            ;; Morris Left

            ld de,#CPCT_VMEM_START_ASM
            ld c,#10
            ld b,#85
            call cpct_getScreenPtr_asm

            ex de,hl                

            ld hl,#_spr_morris_grande_0   ;; sprite a dibujar
            ld c,#SPR_MORRIS_GRANDE_W
            ld b,#SPR_MORRIS_GRANDE_H
            call cpct_drawSprite_asm


            ;; Morris Right

            ld de,#CPCT_VMEM_START_ASM
            ld c,#54
            ld b,#85
            call cpct_getScreenPtr_asm

            ex de,hl                

            ld hl,#_spr_morris_grande_1   ;; sprite a dibujar
            ld c,#SPR_MORRIS_GRANDE_W
            ld b,#SPR_MORRIS_GRANDE_H
            call cpct_drawSprite_asm
           

        ret
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Erase Sprite with solid Box
    sys_render_erase_Sprite::
        ld de,#CPCT_VMEM_START_ASM
        ld c,e_posX(ix)
        ld b,e_posY(ix)
        call cpct_getScreenPtr_asm
        ex de,hl
        ld c,e_width(ix)
        ld b,e_height(ix)
        xor a
        call cpct_drawSolidBox_asm
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Render Controls
    sys_render_teclapulsada::
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra o
                ld de,#CPCT_VMEM_START_ASM
                ld c,#17
                ld b,#60
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_6   ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm

                ld de,#CPCT_VMEM_START_ASM
                ld c,#15
                ld b,#80
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_1   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm

                ld de,#CPCT_VMEM_START_ASM
                ld c,#50
                ld b,#70
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_morris_grande_1 ;; sprite a dibujar
                ld c,#SPR_MORRIS_GRANDE_W
                ld b,#SPR_MORRIS_GRANDE_H
                call cpct_drawSprite_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra p
                ld de,#CPCT_VMEM_START_ASM
                ld c,#30
                ld b,#60
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_3  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm

                ld de,#CPCT_VMEM_START_ASM
                ld c,#30
                ld b,#80
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_0  ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;letra Q

                ld de,#CPCT_VMEM_START_ASM
                ld c,#18
                ld b,#120
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_4   ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm



                ld de,#CPCT_VMEM_START_ASM
                ld c,#28
                ld b,#120
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_1   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm


                ld de,#CPCT_VMEM_START_ASM
                ld c,#50
                ld b,#130
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_morris_grande_2  ;; sprite a dibujar
                ld c,#SPR_MORRIS_GRANDE_W
                ld b,#SPR_MORRIS_GRANDE_H
                call cpct_drawSprite_asm




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra A

                ld de,#CPCT_VMEM_START_ASM
                ld c,#18
                ld b,#140
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_1  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm



                ld de,#CPCT_VMEM_START_ASM
                ld c,#28
                ld b,#140
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_0   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ret


    sys_render_teclanopulsada::
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra o
                ld de,#CPCT_VMEM_START_ASM
                ld c,#17
                ld b,#60
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_2  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H 
                call cpct_drawSprite_asm

                ld de,#CPCT_VMEM_START_ASM
                ld c,#15
                ld b,#80
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_0   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;, letra p
                ld de,#CPCT_VMEM_START_ASM
                ld c,#30
                ld b,#60
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_7  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm



                ld de,#CPCT_VMEM_START_ASM
                ld c,#30
                ld b,#80
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_1   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm

                ld de,#CPCT_VMEM_START_ASM
                ld c,#50
                ld b,#70
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_morris_grande_0 ;; sprite a dibujar
                ld c,#SPR_MORRIS_GRANDE_W
                ld b,#SPR_MORRIS_GRANDE_H
                call cpct_drawSprite_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra Q

                ld de,#CPCT_VMEM_START_ASM
                ld c,#18
                ld b,#120
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_0  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm



                ld de,#CPCT_VMEM_START_ASM
                ld c,#28
                ld b,#120
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_0   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; letra A

                ld de,#CPCT_VMEM_START_ASM
                ld c,#18
                ld b,#140
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_keys_5  ;; sprite a dibujar
                ld c,#SPR_MARCO_KEY_W 
                ld b,#SPR_MARCO_KEY_H
                call cpct_drawSprite_asm



                ld de,#CPCT_VMEM_START_ASM
                ld c,#28
                ld b,#140
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_tecla_1   ;; sprite a dibujar
                ld c,#SPR_KEY_W
                ld b,#SPR_KEY_H
                call cpct_drawSprite_asm


                ld de,#CPCT_VMEM_START_ASM
                ld c,#50
                ld b,#130
                call cpct_getScreenPtr_asm

                ex de,hl                ;; donde dibujo

                ld hl,#_spr_morris_grande_3  ;; sprite a dibujar
                ld c,#SPR_MORRIS_GRANDE_W
                ld b,#SPR_MORRIS_GRANDE_H
                call cpct_drawSprite_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ret